@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Sun Microsystems Inc.
+# Copyright 2005-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -92,6 +92,8 @@ class BrailleGenerator:
              self._getBrailleRegionsForPageTab
         self.brailleGenerators[rolenames.ROLE_PAGE_TAB_LIST]       = \
              self._getBrailleRegionsForPageTabList
+        self.brailleGenerators[rolenames.ROLE_PARAGRAPH]           = \
+             self._getBrailleRegionsForText
         self.brailleGenerators[rolenames.ROLE_PASSWORD_TEXT]       = \
              self._getBrailleRegionsForText
         self.brailleGenerators[rolenames.ROLE_PROGRESS_BAR]        = \
@@ -262,8 +264,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getDefaultBrailleRegions", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
 
@@ -274,9 +275,10 @@ class BrailleGenerator:
             text += value
 
         if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if len(text):
-                text += " "
-            text += getBrailleForRoleName(obj)
+            if (obj.label == None) or (obj.role != rolenames.ROLE_UNKNOWN):
+                if len(text):
+                    text += " "
+                text += getBrailleForRoleName(obj)
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -311,8 +313,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForAnimation", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
 
@@ -363,8 +364,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForCheckBox", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
         if len(text):
@@ -397,8 +397,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForCheckMenuItem", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
         if len(text):
@@ -452,8 +451,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForComboBox", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         regions = []
         regions.append(braille.Region(obj.label + " "))
@@ -467,6 +465,9 @@ class BrailleGenerator:
         #
         # Preference is given to #1, if it exists.
         #
+        # If the label of the combo box is the same as the utterance for
+        # the child object, then this utterance is only displayed once.
+        #
         # [[[TODO: WDW - Combo boxes are complex beasts.  This algorithm
         # needs serious work.  Logged as bugzilla bug 319745.]]]
         #
@@ -480,7 +481,10 @@ class BrailleGenerator:
                 textObj = child
 
         if textObj:
-            regions.append(braille.Text(textObj))
+            result = atspi.getTextLineAtCaret(textObj)
+            line = result[0]
+            if line != obj.label:
+                regions.append(braille.Text(textObj))
         else:
             selectedItem = None
             comboSelection = obj.selection
@@ -488,10 +492,14 @@ class BrailleGenerator:
                 selectedItem = atspi.Accessible.makeAccessible(\
                     comboSelection.getSelectedChild(0))
             if selectedItem:
-                regions.append(braille.Region(selectedItem.label))
+                if selectedItem.label != obj.label:
+                    regions.append(braille.Region(selectedItem.label))
             else:
                 if obj.text:
-                    regions.append(braille.Text(obj))
+                    result = atspi.getTextLineAtCaret(obj)
+                    selectedText = result[0]
+                    if selectedText != obj.label:
+                        regions.append(braille.Text(obj))
                 else:
                     debug.println(
                         debug.LEVEL_SEVERE,
@@ -604,8 +612,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForIcon", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
 
@@ -685,8 +692,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForMenu", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
 
@@ -735,8 +741,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForMenuItem", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
 
@@ -813,8 +818,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForPageTab", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
 
@@ -887,8 +891,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForRadioButton", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
         if len(text):
@@ -921,8 +924,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForRadioMenuItem", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         text = obj.label
         if len(text):
@@ -1151,8 +1153,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForTerminal", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
         label = None
         frame = atspi.getFrame(obj)
@@ -1319,7 +1320,15 @@ class BrailleGenerator:
                     result = self.getBrailleRegions(child, False)
                     regions.extend(result[0])
 
-                    if selection and selection.isChildSelected(i):
+                    # This helps us determine which child is the selected
+                    # child.  Tracking the SELECTED state is not always
+                    # useful (it seems to be inconsistently handled by
+                    # toolkits), so we look at the parent's selection model.
+                    # In addition, we add a STATE_ARMED check here as a
+                    # workaround to the way OOo handles its menu items.
+                    #
+                    if (selection and selection.isChildSelected(i)) \
+                       or child.state.count(atspi.Accessibility.STATE_ARMED):
                         selectedRegion = result[1]
 
         return [regions, selectedRegion]
@@ -1334,12 +1343,9 @@ class BrailleGenerator:
         Returns a list of Regions to display.
         """
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        verbosity = settings.brailleVerbosityLevel
 
-        brailleRolenameStyle = settings.getSetting(
-            settings.BRAILLE_ROLENAME_STYLE,
-            settings.BRAILLE_ROLENAME_STYLE_LONG)
+        brailleRolenameStyle = settings.brailleRolenameStyle
 
         regions = []
         parent = obj.parent
